/* 
Script de creacin del esquema de la BBDD (tablas y secuencias).
PFC - Base de dades
Autor: Alfredo del Valle Lpez
*/
-- Eliminar tablas a crear si existen previamente

Drop Table TipoPrueba Cascade Constraints;
Drop Table Diagnostico Cascade Constraints;
Drop Table TipoAlergia Cascade Constraints;
Drop Table Provincia Cascade Constraint;
Drop Table TipoCentro Cascade Constraints;
Drop Table Centro Cascade Constraints;
Drop Table Proveedor Cascade Constraints;
Drop Table Farmaco Cascade Constraints;
Drop Table Servicio Cascade Constraints;
Drop Table ServicioCentro Cascade Constraints;
Drop Table Botica Cascade Constraints;
Drop Table MedioCita Cascade Constraints;
Drop Table Enfermero Cascade Constraints;
Drop Table Farmaceutico Cascade Constraints;
Drop Table Medico Cascade Constraints;
Drop Table Paciente Cascade Constraints;
Drop Table Alergia Cascade Constraints;
Drop Table Patologia Cascade Constraints;
Drop Table Cita Cascade Constraints;
Drop Table Prueba Cascade Constraints;
Drop Table Tratamiento Cascade Constraints;
Drop Table Administracion Cascade Constraints;
Drop Table Pedido Cascade Constraints;
Drop Table Logs Cascade Constraints;

-- Eliminar secuencias a crear si existen previamente

Drop Sequence Seq_TipoPrueba;
Drop Sequence Seq_Diagnostico;
Drop Sequence Seq_TipoAlergia;
Drop Sequence Seq_TipoCentro;
Drop Sequence Seq_Centro;
Drop Sequence Seq_Proveedor;
Drop Sequence Seq_Farmaco;
Drop Sequence Seq_Servicio;
Drop Sequence Seq_ServicioCentro;
Drop Sequence Seq_Botica;
Drop Sequence Seq_MedioCita;
Drop Sequence Seq_Enfermero;
Drop Sequence Seq_Farmaceutico;
Drop Sequence Seq_Medico;
Drop Sequence Seq_Paciente;
Drop Sequence Seq_Alergia;
Drop Sequence Seq_Patologia;
Drop Sequence Seq_Cita;
Drop Sequence Seq_Prueba;
Drop Sequence Seq_Tratamiento;
Drop Sequence Seq_Administracion;
Drop Sequence Seq_Pedido;
Drop Sequence Seq_Logs;

--Crear tablas

Create Table TipoPrueba (
    ID Integer Constraint PK_TIPOPRUEBA Primary Key,
    denominacion Varchar(70 Char) Not Null   
);

Create Table Diagnostico (
    ID Integer Constraint PK_DIAGNOSTICO Primary Key,
    denominacion Varchar(100 Char) Not Null,
    capitulo Varchar(200 Char) Not Null,
    seccion Varchar(180 Char) Not Null,
    subseccion Varchar(180 Char) Not Null	
);

Create Table TipoAlergia (
    ID Integer Constraint PK_TIPOALERGIA Primary Key,
    denominacion Varchar(50 Char) Not Null   
);

Create Table Provincia (
    ID Integer Constraint PK_PROVINCIA Primary Key,
	denominacion Varchar(50 Char) Not Null   
);

Create Table TipoCentro (
    ID Integer Constraint PK_TIPOCENTRO Primary Key,
	denominacion Varchar(50 Char) Not Null
);

Create Table Centro (
    ID Integer Constraint PK_CENTRO Primary Key,
	provincia Integer Constraint FK1_CENTRO REFERENCES Provincia(ID) Not Null,
	tipo Integer Constraint FK2_CENTRO REFERENCES TipoCentro(ID) Not Null,
    denominacion Varchar(100 Char) Not Null
);

Create Table Proveedor (
    ID Integer Constraint PK_PROVEEDOR Primary Key,
    CIF Varchar(10 Char) Not null CONSTRAINT AK_PROVEEDOR UNIQUE, 
    denominacion Varchar(50 Char) Not Null,
    direccion Varchar(75 Char),
    cp Varchar(10 Char), 
    telefono Varchar(10 Char),
    email Varchar(50 Char) 
);

Create Table Farmaco (
    ID Integer Constraint PK_FARMACO Primary Key,
    denominacion Varchar(100 Char) Not Null,
    principio Varchar(75 Char) Not Null,
    grupo Varchar(50 Char) Not Null
);
 
Create Table Servicio(
    ID Integer Constraint PK_SERVICIO Primary Key,    
    denominacion Varchar(50 Char) Not Null,
    codigo Varchar(4 Char) Not Null
);

Create Table ServicioCentro(
    ID Integer Constraint PK_SERVICIOCENTRO Primary Key,    
    servicio Integer Constraint FK1_SERVICIOCENTRO REFERENCES Servicio(ID) Not Null,
    centro Integer Constraint FK2_SERVICIOCENTRO REFERENCES Centro(ID) Not Null
);

Create Table Botica (
    ID Integer Constraint PK_BOTICA Primary Key,
    farmaco Integer Constraint FK1_BOTICA REFERENCES Farmaco(ID) Not Null,
	centro Integer Constraint FK2_BOTICA REFERENCES ServicioCentro(ID) Not Null,
    cantidad Integer Not Null,
	minimo Integer Not Null,
	caducidad Date Not Null,
	constraint check_stock check(cantidad - minimo > -1)  
);

Create Table MedioCita(
    ID Integer Constraint PK_MEDIOCITA Primary Key, 
    denominacion Varchar(50 Char) Not Null
);

Create Table Enfermero(
    ID Integer Constraint PK_ENFERMERO Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_ENFERMERO UNIQUE,
    centro Integer Constraint FK_ENFERMERO References ServicioCentro(ID) Not Null,	
    nombre Varchar(50 Char) Not Null,
    apellido1 Varchar(50 Char) Not Null,
	apellido2 Varchar(50 Char),
    telefono Varchar(10 Char)
);

Create Table Farmaceutico(
    ID Integer Constraint PK_FARMACEUTICO Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_FARMACEUTICO UNIQUE,
    centro Integer Constraint FK_FARMACEUTICO References ServicioCentro(ID) Not Null,
    nombre Varchar(50 Char) Not Null,
    apellido1 Varchar(50 Char) Not Null,
	apellido2 Varchar(50 Char),
    telefono Varchar(10 Char)
);

Create Table Medico(
    ID Integer Constraint PK_MEDICO Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_MEDICO UNIQUE,
    centro Integer Constraint FK_MEDICO References ServicioCentro(ID) Not Null,	
    nombre Varchar(50 Char) Not Null,
    apellido1 Varchar(50 Char) Not Null,
	apellido2 Varchar(50 Char),
    telefono Varchar(10 Char)
);

Create Table Paciente(
    ID Integer Constraint PK_PACIENTE Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_PACIENTE UNIQUE,
    provincia Integer Constraint FK_PACIENTE References Provincia(ID) Not Null,	
    nombre Varchar(50 Char) Not Null,
    apellido1 Varchar(50 Char) Not Null,
	apellido2 Varchar(50 Char),
    telefono Varchar(10 Char)
);

Create Table Alergia(
    ID Integer Constraint PK_ALERGIA Primary Key,
    paciente Integer Constraint FK1_ALERGIA References Paciente(ID) Not Null,
    tipo Integer Constraint FK2_ALERGIA References TipoAlergia(ID) Not Null,	
    fecha Date Not Null
);
	
Create Table Patologia(
    ID Integer Constraint PK_PATOLOGIA Primary Key,
    paciente Integer Constraint FK1_PATOLOGIA References Paciente(ID) Not Null,
    diagnostico Integer Constraint FK2_PATOLOGIA References Diagnostico(ID) Not Null,	
    fechaInicio Date Not Null,
    fechaFin Date,
    baja number(1) check(baja in (0, 1)) Not Null
);

Create Table Cita(
    ID Integer Constraint PK_CITA Primary Key,
    medico Integer Constraint FK1_CITA References Medico(ID) Not Null,
    enfermero Integer Constraint FK2_CITA References Enfermero(ID) Not Null,
    paciente Integer Constraint FK3_CITA References Paciente(ID) Not Null,	
    medio Integer Constraint FK4_CITA References MedioCita(ID) Not Null,	
    patologia Integer Constraint FK5_CITA References Patologia(ID) Not Null,	
    fechaHora Date Not Null,
    asistencia number(1) check(asistencia in (0, 1)) Not Null,
    urgencia number(1) check(urgencia in (0, 1)) Not Null
);

Create Table Prueba(
    ID Integer Constraint PK_PRUEBA Primary Key,
    patologia Integer Constraint FK1_PRUEBA References Patologia(ID) Not Null,
    tipo Integer Constraint FK2_PRUEBA References TipoPrueba(ID) Not Null,	
    fechaHora Date Not Null,
	resultado Varchar(150 Char)
);

Create Table Tratamiento(
    ID Integer Constraint PK_TRATAMIENTO Primary Key,
    patologia Integer Constraint FK1_TRATAMIENTO References Patologia(ID) Not Null,
    farmaco Integer Constraint FK2_TRATAMIENTO References Farmaco(ID) Not Null,
	medico Integer Constraint FK3_TRATAMIENTO References Medico(ID) Not Null,
	fechaInicio Date Not Null,    
	fechaFin Date Not Null,
	periodicidad Integer Not Null,
	cantidad Integer Not Null
);

Create Table Administracion(
    ID Integer Constraint PK_ADMINISTRACION Primary Key,
    enfermero Integer Constraint FK1_ADMINISTRACION References Enfermero(ID) Not Null,
    tratamiento Integer Constraint FK2_ADMINISTRACION References Tratamiento(ID) Not Null,	
	fecha Date Not Null,    
	periodo Integer Not Null,
	cantidad Integer Not Null
);

Create Table Pedido(
    ID Integer Constraint PK_PEDIDO Primary Key,
    farmaceutico Integer Constraint FK1_PEDIDO References Farmaceutico(ID) Not Null,
    farmaco Integer Constraint FK2_PEDIDO References Farmaco(ID) Not Null,
    proveedor Integer Constraint FK3_PEDIDO References Proveedor(ID) Not Null,	
    cantidad Integer Not Null,
	fechaHora Date Not Null
);

Create Table Logs(
    ID Integer Constraint PK_LOGS Primary Key,
    procedimiento Varchar2(50),
    error Integer,
	descripcion Varchar2(1024),
	fecha Date Not Null
);

--Crear secuencias
        
Create Sequence Seq_TipoPrueba Increment By 1 Start With 1;
Create Sequence Seq_Diagnostico Increment By 1 Start With 1;
Create Sequence Seq_TipoAlergia Increment By 1 Start With 1;
Create Sequence Seq_TipoCentro Increment By 1 Start With 1;
Create Sequence Seq_Centro Increment By 1 Start With 1;
Create Sequence Seq_Proveedor Increment By 1 Start With 1;
Create Sequence Seq_Farmaco Increment By 1 Start With 1;
Create Sequence Seq_Servicio Increment By 1 Start With 1;
Create Sequence Seq_ServicioCentro Increment By 1 Start With 1;
Create Sequence Seq_Botica Increment By 1 Start With 1;
Create Sequence Seq_MedioCita Increment By 1 Start With 1;
Create Sequence Seq_Enfermero Increment By 1 Start With 1;
Create Sequence Seq_Farmaceutico Increment By 1 Start With 1;
Create Sequence Seq_Medico Increment By 1 Start With 1;
Create Sequence Seq_Paciente Increment By 1 Start With 1;
Create Sequence Seq_Alergia Increment By 1 Start With 1;
Create Sequence Seq_Patologia Increment By 1 Start With 1;
Create Sequence Seq_Cita Increment By 1 Start With 1;
Create Sequence Seq_Prueba Increment By 1 Start With 1;
Create Sequence Seq_Tratamiento Increment By 1 Start With 1;
Create Sequence Seq_Administracion Increment By 1 Start With 1;
Create Sequence Seq_Pedido Increment By 1 Start With 1;
Create Sequence Seq_Logs Increment By 1 Start With 1;

--Iniciar secuencias

Create Or Replace Trigger Insert_ID_TipoPrueba
    Before Insert On TipoPrueba
    For Each Row
Begin
   Select Seq_TipoPrueba.Nextval Into :New.ID
   From Dual;
End Insert_ID_TipoPrueba;
/

Create Or Replace Trigger Insert_ID_Diagnostico
    Before Insert On Diagnostico
    For Each Row
Begin
   Select Seq_Diagnostico.Nextval Into :New.ID
   From Dual;
End Insert_ID_Diagnostico;
/

Create Or Replace Trigger Insert_ID_TipoAlergia
    Before Insert On TipoAlergia
    For Each Row
Begin
   Select Seq_TipoAlergia.Nextval Into :New.ID
   From Dual;
End Insert_ID_TipoAlergia;
/

Create Or Replace Trigger Insert_ID_TipoCentro
    Before Insert On TipoCentro
    For Each Row
Begin
   Select Seq_TipoCentro.Nextval Into :New.ID
   From Dual;
End Insert_ID_TipoCentro;
/

Create Or Replace Trigger Insert_ID_Centro
    Before Insert On Centro
    For Each Row
Begin
   Select Seq_Centro.Nextval Into :New.ID
   From Dual;
End Insert_ID_Centro;
/

Create Or Replace Trigger Insert_ID_Proveedor
    Before Insert On Proveedor
    For Each Row
Begin
   Select Seq_Proveedor.Nextval Into :New.ID
   From Dual;
End Insert_ID_Proveedor;
/

Create Or Replace Trigger Insert_ID_Farmaco
    Before Insert On Farmaco
    For Each Row
Begin
   Select Seq_Farmaco.Nextval Into :New.ID
   From Dual;
End Insert_ID_Farmaco;
/

Create Or Replace Trigger Insert_ID_Servicio
    Before Insert On Servicio
    For Each Row
Begin
   Select Seq_Servicio.Nextval Into :New.ID
   From Dual;
End Insert_ID_Servicio;
/

Create Or Replace Trigger Insert_ID_ServicioCentro
    Before Insert On ServicioCentro
    For Each Row
Begin
   Select Seq_ServicioCentro.Nextval Into :New.ID
   From Dual;
End Insert_ID_ServicioCentro;
/

Create Or Replace Trigger Insert_ID_Botica
    Before Insert On Botica
    For Each Row
Begin
   Select Seq_Botica.Nextval Into :New.ID
   From Dual;
End Insert_ID_Botica;
/

Create Or Replace Trigger Insert_ID_MedioCita
    Before Insert On MedioCita
    For Each Row
Begin
   Select Seq_MedioCita.Nextval Into :New.ID
   From Dual;
End Insert_ID_MedioCita;
/

Create Or Replace Trigger Insert_ID_Enfermero
    Before Insert On Enfermero
    For Each Row
Begin
   Select Seq_Enfermero.Nextval Into :New.ID
   From Dual;
End Insert_ID_Enfermero;
/

Create Or Replace Trigger Insert_ID_Farmaceutico
    Before Insert On Farmaceutico
    For Each Row
Begin
   Select Seq_Farmaceutico.Nextval Into :New.ID
   From Dual;
End Insert_ID_Farmaceutico;
/

Create Or Replace Trigger Insert_ID_Medico
    Before Insert On Medico
    For Each Row
Begin
   Select Seq_Medico.Nextval Into :New.ID
   From Dual;
End Insert_ID_Medico;
/

Create Or Replace Trigger Insert_ID_Paciente
    Before Insert On Paciente
    For Each Row
Begin
   Select Seq_Paciente.Nextval Into :New.ID
   From Dual;
End Insert_ID_Paciente;
/

Create Or Replace Trigger Insert_ID_Alergia
    Before Insert On Alergia
    For Each Row
Begin
   Select Seq_Alergia.Nextval Into :New.ID
   From Dual;
End Insert_ID_Alergia;
/

Create Or Replace Trigger Insert_ID_Patologia
    Before Insert On Patologia
    For Each Row
Begin
   Select Seq_Patologia.Nextval Into :New.ID
   From Dual;
End Insert_ID_Patologia;
/

Create Or Replace Trigger Insert_ID_Cita
    Before Insert On Cita
    For Each Row
Begin
   Select Seq_Cita.Nextval Into :New.ID
   From Dual;
End Insert_ID_Cita;
/

Create Or Replace Trigger Insert_ID_Prueba
    Before Insert On Prueba
    For Each Row
Begin
   Select Seq_Prueba.Nextval Into :New.ID
   From Dual;
End Insert_ID_Prueba;
/

Create Or Replace Trigger Insert_ID_Tratamiento
    Before Insert On Tratamiento
    For Each Row
Begin
   Select Seq_Tratamiento.Nextval Into :New.ID
   From Dual;
End Insert_ID_Tratamiento;
/

Create Or Replace Trigger Insert_ID_Administracion
    Before Insert On Administracion
    For Each Row
Begin
   Select Seq_Administracion.Nextval Into :New.ID
   From Dual;
End Insert_ID_Administracion;
/

Create Or Replace Trigger Insert_ID_Pedido
    Before Insert On Pedido
    For Each Row
Begin
   Select Seq_Pedido.Nextval Into :New.ID
   From Dual;
End Insert_ID_Pedido;
/

Create Or Replace Trigger Insert_ID_Logs
    Before Insert On Logs
    For Each Row
Begin
   Select Seq_Logs.Nextval Into :New.ID
   From Dual;
End Insert_ID_Logs;
/