/* 
Script de creacin de los Triggers de la BBDD.
PFC - Base de dades
Autor: Alfredo del Valle Lpez
*/

-- Creacin del Trigger que controla el stock de la Botica en funcin de los pedidos a un proveedor.

Create or Replace Trigger check_pedido_botica
After Insert or Delete or Update on Pedido for each row
begin
	if(inserting or updating) then
		Update Botica Set cantidad = cantidad + :new.cantidad Where Botica.farmaco = :new.farmaco 
      And Botica.centro = (Select centro From Farmaceutico where ID = :new.farmaceutico);
		if (sql%rowcount = 0) then
			Insert Into Botica(farmaco,centro,cantidad,minimo,caducidad) 
				Values (:new.farmaco,(Select centro From Farmaceutico where ID = :new.farmaceutico),:new.cantidad,15,SYSDATE+180);
		end if;
	end if;
	if(updating or deleting) then
		Update Botica Set cantidad = cantidad - :old.cantidad Where Botica.farmaco = :old.farmaco 
      And Botica.centro = (Select centro From Farmaceutico where ID = :old.farmaceutico);
	end if;
end check_pedido_botica;
/

-- Creacin del Trigger que controla el stock de la Botica en funcin de los tratamientos recetados al paciente.

Create or Replace Trigger check_tratamiento_botica
Before Insert or Delete or Update on Tratamiento for each row
begin
	if(inserting or updating) then
		Update Botica Set cantidad = (cantidad - (:new.cantidad * :new.periodicidad * (:new.fechaFin - :new.fechaInicio))) 
			Where Botica.farmaco = :new.farmaco And Botica.centro = 
        (Select ID From ServicioCentro Where servicio = 14 And centro = 
          (Select centro From ServicioCentro Where ID = (Select centro From Medico where ID = :new.medico)));
		if (sql%rowcount = 0) then
			RAISE_APPLICATION_ERROR(-20000, 'Error: Se est intentando dar un tratamiento con un frmaco no disponible en la botica.');
		end if;
	end if;
	if(updating or deleting) then
		Update Botica Set cantidad = (cantidad + (:old.cantidad * :old.periodicidad * (:old.fechaFin - :old.fechaInicio))) 
			Where Botica.farmaco = :old.farmaco And Botica.centro = 
        (Select ID From ServicioCentro Where servicio = 14 And centro = 
          (Select centro From ServicioCentro Where ID = (Select centro From Medico where ID = :old.medico)));
		if (sql%rowcount = 0) then
			Insert Into Botica(farmaco,centro,cantidad,minimo,caducidad) 
				Values (:old.farmaco,(Select ID From ServicioCentro Where servicio = 14 And centro = 
          (Select centro From ServicioCentro Where ID = (Select centro From Medico where ID = :old.medico))), 
            :old.cantidad,15,SYSDATE+180);
		end if;
	end if;
end check_tratamiento_botica;
/

-- Creacin del Trigger que confirma la asistencia de una cita automaticamente cuando se trata de una urgencia.

Create or Replace Trigger set_asistencia_cita
Before Insert or Update on Cita for each row
begin
	if(inserting or updating) then
		if (:new.urgencia = 1) then 
			:new.asistencia := 1;
		end if;
	end if;
end set_asistencia_cita;
/



