/* 
Script de creacin de los Procedimientos de la BBDD.
PFC - Base de dades
Autor: Alfredo del Valle Lpez
*/

-- Creacin del Procedimiento que aade un Paciente en la BD.

Create Or Replace Procedure altaPaciente (
  P_DNI In Paciente.DNI%Type, 
  P_provincia In Paciente.provincia%Type, 
  P_nombre In Paciente.nombre%Type, 
  P_apellido1 In Paciente.apellido1%Type, 
  P_apellido2 In Paciente.apellido2%Type default null,
  P_telefono In Paciente.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;

Begin
resultado:='Alta Paciente con DNI:' || P_DNI || ' correcta.';
Insert Into Paciente(DNI,provincia,nombre,apellido1,apellido2,telefono) Values (P_DNI,P_provincia,P_nombre,P_apellido1,P_apellido2,P_telefono);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: El Paciente ya existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica un Paciente en la BD

Create Or Replace Procedure  modPaciente (
  P_ID In Paciente.ID%Type,
  P_DNI In Paciente.DNI%Type, 
  P_provincia In Paciente.provincia%Type, 
  P_nombre In Paciente.nombre%Type, 
  P_apellido1 In Paciente.apellido1%Type, 
  P_apellido2 In Paciente.apellido2%Type default null,
  P_telefono In Paciente.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
resultado:='Modificacin de Paciente con cdigo:' || P_ID || ' correcta.';
Update Paciente Set DNI= P_DNI, provincia= P_provincia, nombre= P_nombre, apellido1= P_apellido1, apellido2= P_apellido2, telefono= P_telefono Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Paciente no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina un Paciente de la BD.

Create Or Replace Procedure bajaPaciente (
  P_ID In Paciente.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
Resultado:='Baja de Paciente con cdigo:' || P_ID || ' correcta.';
Delete From Paciente Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Paciente no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade un Enfermero en la BD.

Create Or Replace Procedure altaEnfermero (
  P_DNI In Enfermero.DNI%Type, 
  P_centro In Enfermero.centro%Type, 
  P_nombre In Enfermero.nombre%Type, 
  P_apellido1 In Enfermero.apellido1%Type, 
  P_apellido2 In Enfermero.apellido2%Type default null,
  P_telefono In Enfermero.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;

Begin
resultado:='Alta Enfermero con DNI:' || P_DNI || ' correcta.';
Insert Into Enfermero(DNI,centro,nombre,apellido1,apellido2,telefono) Values (P_DNI,P_centro,P_nombre,P_apellido1,P_apellido2,P_telefono);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: El Enfermero ya existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica un Enfermero en la BD

Create Or Replace Procedure  modEnfermero (
  P_ID In Enfermero.ID%Type,
  P_DNI In Enfermero.DNI%Type, 
  P_centro In Enfermero.centro%Type, 
  P_nombre In Enfermero.nombre%Type, 
  P_apellido1 In Enfermero.apellido1%Type, 
  P_apellido2 In Enfermero.apellido2%Type default null,
  P_telefono In Enfermero.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
resultado:='Modificacin de Enfermero con cdigo:' || P_ID || ' correcta.';
Update Enfermero Set DNI= P_DNI, centro= P_centro, nombre= P_nombre, apellido1= P_apellido1, apellido2= P_apellido2, telefono= P_telefono Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Enfermero no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina un Enfermero de la BD.

Create Or Replace Procedure bajaEnfermero (
  P_ID In Enfermero.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
Resultado:='Baja de Enfermero con cdigo:' || P_ID || ' correcta.';
Delete From Enfermero Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Enfermero no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade un Medico en la BD.

Create Or Replace Procedure altaMedico (
  P_DNI In Medico.DNI%Type, 
  P_centro In Medico.centro%Type, 
  P_nombre In Medico.nombre%Type, 
  P_apellido1 In Medico.apellido1%Type, 
  P_apellido2 In Medico.apellido2%Type default null,
  P_telefono In Medico.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;

Begin
resultado:='Alta Medico con DNI:' || P_DNI || ' correcta.';
Insert Into Medico(DNI,centro,nombre,apellido1,apellido2,telefono) Values (P_DNI,P_centro,P_nombre,P_apellido1,P_apellido2,P_telefono);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: El Medico ya existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica un Medico en la BD

Create Or Replace Procedure  modMedico (
  P_ID In Medico.ID%Type,
  P_DNI In Medico.DNI%Type, 
  P_centro In Medico.centro%Type, 
  P_nombre In Medico.nombre%Type, 
  P_apellido1 In Medico.apellido1%Type, 
  P_apellido2 In Medico.apellido2%Type default null,
  P_telefono In Medico.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
resultado:='Modificacin de Medico con cdigo:' || P_ID || ' correcta.';
Update Medico Set DNI= P_DNI, centro= P_centro, nombre= P_nombre, apellido1= P_apellido1, apellido2= P_apellido2, telefono= P_telefono Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Medico no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina un Medico de la BD.

Create Or Replace Procedure bajaMedico (
  P_ID In Medico.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
Resultado:='Baja de Medico con cdigo:' || P_ID || ' correcta.';
Delete From Medico Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Medico no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade un Farmaceutico en la BD.

Create Or Replace Procedure altaFarmaceutico (
  P_DNI In Farmaceutico.DNI%Type, 
  P_centro In Farmaceutico.centro%Type, 
  P_nombre In Farmaceutico.nombre%Type, 
  P_apellido1 In Farmaceutico.apellido1%Type, 
  P_apellido2 In Farmaceutico.apellido2%Type default null,
  P_telefono In Farmaceutico.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;

Begin
resultado:='Alta Farmaceutico con DNI:' || P_DNI || ' correcta.';
Insert Into Farmaceutico(DNI,centro,nombre,apellido1,apellido2,telefono) Values (P_DNI,P_centro,P_nombre,P_apellido1,P_apellido2,P_telefono);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: El Farmaceutico ya existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica un Farmaceutico en la BD

Create Or Replace Procedure  modFarmaceutico (
  P_ID In Farmaceutico.ID%Type,
  P_DNI In Farmaceutico.DNI%Type, 
  P_centro In Farmaceutico.centro%Type,   
  P_nombre In Farmaceutico.nombre%Type, 
  P_apellido1 In Farmaceutico.apellido1%Type, 
  P_apellido2 In Farmaceutico.apellido2%Type default null,
  P_telefono In Farmaceutico.telefono%Type default null, 
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
resultado:='Modificacin de Farmaceutico con cdigo:' || P_ID || ' correcta.';
Update Farmaceutico Set DNI= P_DNI, centro= P_centro, nombre= P_nombre, apellido1= P_apellido1, apellido2= P_apellido2, telefono= P_telefono Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Farmaceutico no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina un Farmaceutico de la BD.

Create Or Replace Procedure bajaFarmaceutico (
  P_ID In Farmaceutico.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
Resultado:='Baja de Farmaceutico con cdigo:' || P_ID || ' correcta.';
Delete From Farmaceutico Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Farmaceutico no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade un Proveedor en la BD.

Create Or Replace Procedure altaProveedor (
  P_CIF In Proveedor.CIF%Type, 
  P_denominacion In Proveedor.denominacion%Type, 
  P_direccion In Proveedor.direccion%Type default null, 
  P_cp In Proveedor.cp%Type default null, 
  P_telefono In Proveedor.telefono%Type default null, 
  P_email In Proveedor.email%Type default null,
  resultado Out Nocopy Varchar)
Is
P_error Integer;

Begin
resultado:='Alta Proveedor con CIF:' || P_CIF || ' correcta.';
Insert Into Proveedor(CIF,denominacion,direccion,cp,telefono,email) Values (P_CIF,P_denominacion,P_direccion,P_cp,P_telefono,P_email);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: El Proveedor ya existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica un Proveedor en la BD

Create Or Replace Procedure  modProveedor (
  P_ID In Proveedor.ID%Type,
  P_CIF In Proveedor.CIF%Type, 
  P_denominacion In Proveedor.denominacion%Type, 
  P_direccion In Proveedor.direccion%Type default null, 
  P_cp In Proveedor.cp%Type default null, 
  P_telefono In Proveedor.telefono%Type default null, 
  P_email In Proveedor.email%Type default null,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
resultado:='Modificacin de Proveedor con cdigo:' || P_ID || ' correcta.';
Update Proveedor Set CIF= P_CIF, denominacion= P_denominacion, direccion= P_direccion, cp= P_cp, telefono= P_telefono, email= P_email Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Proveedor no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina un Proveedor de la BD.

Create Or Replace Procedure bajaProveedor (
  P_ID In Proveedor.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;

Begin
Resultado:='Baja de Proveedor con cdigo:' || P_ID || ' correcta.';
Delete From Proveedor Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Proveedor no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade un Pedido de un Farmacetico en la BD.

Create Or Replace Procedure altaPedido (
  P_farmaceutico In Pedido.farmaceutico%Type, 
  P_farmaco In Pedido.farmaco%Type, 
  P_proveedor In Pedido.proveedor%Type, 
  P_cantidad In Pedido.cantidad%Type, 
  P_fechahora In Pedido.fechahora%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_farmaceutico VARCHAR(10);

Begin
SELECT DNI INTO DNI_farmaceutico FROM farmaceutico WHERE id = P_farmaceutico;
resultado:='Alta Pedido de Farmacetico con DNI: ' || DNI_farmaceutico || ' correcta.';
Insert Into Pedido(farmaceutico,farmaco,proveedor,cantidad,fechahora) Values (P_farmaceutico,P_farmaco,P_proveedor,P_cantidad,P_fechahora);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: El Pedido ya existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica un Pedido de un Farmacetico en la BD

Create Or Replace Procedure modPedido (
  P_ID In Pedido.ID%Type,
  P_farmaceutico In Pedido.farmaceutico%Type, 
  P_farmaco In Pedido.farmaco%Type, 
  P_proveedor In Pedido.proveedor%Type, 
  P_cantidad In Pedido.cantidad%Type, 
  P_fechahora In Pedido.fechahora%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_farmaceutico VARCHAR(10);
P_error_codigo exception;

Begin
SELECT DNI INTO DNI_farmaceutico FROM farmaceutico WHERE id = P_farmaceutico;
resultado:='Modificacin de Pedido de farmacetico con DNI: ' || DNI_farmaceutico || ' correcta.';
Update Pedido Set farmaceutico= P_farmaceutico, farmaco= P_farmaco, proveedor= P_proveedor, cantidad= P_cantidad, fechahora= P_fechahora Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Pedido no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina un Pedido de un Farmacetico de la BD.

Create Or Replace Procedure bajaPedido (
  P_ID In Pedido.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_farmaceutico Integer;
DNI_farmaceutico VARCHAR(10);
P_error_codigo exception;

Begin
SELECT farmaceutico INTO P_farmaceutico FROM Pedido WHERE ID = P_ID;
SELECT DNI INTO DNI_farmaceutico FROM farmaceutico WHERE ID = P_farmaceutico;
resultado:='Eliminacin de Pedido con ID: ' || P_ID || ' del farmacetico con DNI: ' || DNI_farmaceutico || ' correcta.';
Delete From Pedido Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Pedido no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade una Alergia de un Paciente en la BD.

Create Or Replace Procedure altaAlergia (
  P_paciente In Alergia.paciente%Type, 
  P_tipo In Alergia.tipo%Type, 
  P_fecha In Alergia.fecha%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_paciente VARCHAR(10);

Begin
SELECT DNI INTO DNI_paciente FROM Paciente WHERE id = P_paciente;
resultado:='Alta Alergia de Paciente con DNI: ' || DNI_paciente || ' correcta.';
Insert Into Alergia(paciente,tipo,fecha) Values (P_paciente,P_tipo,P_fecha) ;
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: La Alergia ya est registrada para ese Paciente.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica una Alergia de un Paciente en la BD

Create Or Replace Procedure modAlergia (
  P_ID In Alergia.ID%Type,
  P_paciente In Alergia.paciente%Type, 
  P_tipo In Alergia.tipo%Type, 
  P_fecha In Alergia.fecha%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT DNI INTO DNI_paciente FROM paciente WHERE id = P_paciente;
resultado:='Modificacin de Alergia de Paciente con DNI: ' || DNI_paciente || ' correcta.';
Update Alergia Set paciente= P_paciente, tipo= P_tipo, fecha= P_fecha Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Paciente no tiene esta Alergia.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina una Alergia de un Paciente de la BD.

Create Or Replace Procedure bajaAlergia (
  P_ID In Alergia.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT paciente INTO P_paciente FROM Alergia WHERE ID = P_ID;
SELECT DNI INTO DNI_paciente FROM paciente WHERE ID = P_paciente;
resultado:='Eliminacin de Alergia con ID: ' || P_ID || ' del Paciente con DNI: ' || DNI_paciente || ' correcta.';
Delete From Alergia Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: El Paciente no tiene esta Alergia.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade una Patologa de un Paciente en la BD.

Create Or Replace Procedure altaPatologia (
  P_paciente In Patologia.paciente%Type, 
  P_diagnostico In Patologia.diagnostico%Type, 
  P_fechaInicio In Patologia.fechaInicio%Type,
  P_fechaFin In Patologia.fechaFin%Type default null,
  P_baja In Patologia.baja%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_paciente VARCHAR(10);

Begin
SELECT DNI INTO DNI_paciente FROM Paciente WHERE id = P_paciente;
resultado:='Alta Patologia de Paciente con DNI: ' || DNI_paciente || ' correcta.';
Insert Into Patologia(paciente,diagnostico,fechaInicio,fechaFin,baja) Values (P_paciente,P_diagnostico,P_fechaInicio,P_fechaFin,P_baja) ;
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: Ya existe una Patologia con el mismo cdigo para este Paciente.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica una Patologia de un Paciente en la BD

Create Or Replace Procedure modPatologia (
  P_ID In Patologia.ID%Type,
  P_paciente In Patologia.paciente%Type, 
  P_diagnostico In Patologia.diagnostico%Type, 
  P_fechaInicio In Patologia.fechaInicio%Type,
  P_fechaFin In Patologia.fechaFin%Type default null,
  P_baja In Patologia.baja%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT DNI INTO DNI_paciente FROM paciente WHERE id = P_paciente;
resultado:='Modificacin de Patologia de Paciente con DNI: ' || DNI_paciente || ' correcta.';
Update Patologia Set paciente= P_paciente, diagnostico= P_diagnostico, fechaInicio= P_fechaInicio, baja= P_baja Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe ese cdigo de Patologia para este Paciente.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina una Patologa de un Paciente de la BD.

Create Or Replace Procedure bajaPatologia (
  P_ID In Patologia.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT paciente INTO P_paciente FROM Patologia WHERE ID = P_ID;
SELECT DNI INTO DNI_paciente FROM paciente WHERE ID = P_paciente;
resultado:='Eliminacin de Patologa con ID: ' || P_ID || ' del Paciente con DNI: ' || DNI_paciente || ' correcta.';
Delete From Patologia Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe ese cdigo de Patologia para este Paciente.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade una Prueba a la Patologa de un Paciente en la BD.

Create Or Replace Procedure altaPrueba (
  P_patologia In Prueba.patologia%Type, 
  P_tipo In Prueba.tipo%Type, 
  P_fechaHora In Prueba.fechaHora%Type,
  P_resultado In Prueba.resultado%Type default null,
  registro Out Nocopy Varchar)
Is
P_error Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);

Begin
SELECT paciente INTO P_paciente FROM Patologia WHERE ID = P_patologia;
SELECT DNI INTO DNI_paciente FROM Paciente WHERE id = P_paciente;
registro:='Alta Prueba de Paciente con DNI: ' || DNI_paciente || ' para Patologa con ID: ' || P_patologia || ' correcta.';
Insert Into Prueba(patologia,tipo,fechaHora,resultado) Values (P_patologia,P_tipo,P_fechaHora,P_resultado) ;
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, registro, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(registro);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    registro:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(registro);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    registro:='Error!: Ya existe una Prueba con el mismo cdigo para la Patologa de este Paciente.';
    Dbms_output.Put_line(registro);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     registro:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(registro);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica una Prueba de la Patologa de un Paciente en la BD

Create Or Replace Procedure modPrueba (
  P_ID In Prueba.ID%Type,
  P_patologia In Prueba.patologia%Type, 
  P_tipo In Prueba.tipo%Type, 
  P_fechaHora In Prueba.fechaHora%Type,
  P_resultado In Prueba.resultado%Type default null,
  registro Out Nocopy Varchar)
Is
P_error Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT paciente INTO P_paciente FROM Patologia WHERE ID = P_patologia;
SELECT DNI INTO DNI_paciente FROM paciente WHERE id = P_paciente;
registro:='Modificacin de Prueba de Paciente con DNI: ' || DNI_paciente || ' para Patologa con ID: ' || P_patologia || ' correcta.';
Update Prueba Set patologia= P_patologia, tipo= P_tipo, fechaHora= P_fechaHora Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, registro, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(registro);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    registro:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(registro);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    registro:='Error!: No existe una Prueba con el mismo cdigo para la Patologa de este Paciente.';
    Dbms_output.Put_line(registro);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     registro:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(registro);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina una Prueba de una Patologa de un Paciente de la BD.

Create Or Replace Procedure bajaPrueba (
  P_ID In Prueba.ID%Type,
  registro Out Nocopy Varchar)
Is
P_error Integer;
P_patologia Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT patologia INTO P_patologia FROM Prueba WHERE ID = P_ID;
SELECT paciente INTO P_paciente FROM Patologia WHERE ID = P_patologia;
SELECT DNI INTO DNI_paciente FROM paciente WHERE ID = P_paciente;
registro:='Eliminacin de Prueba con ID: ' || P_ID || ' de la Patologa con ID: ' || P_patologia || ' y del Paciente con DNI: ' || DNI_paciente || ' correcta.';
Delete From Prueba Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, registro, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(registro);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    registro:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(registro);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    registro:='Error!: No existe ese cdigo de Prueba para la Patologia de ese Paciente.';
    Dbms_output.Put_line(registro);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     registro:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(registro);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, registro, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade una Cita de un Paciente en la BD.

Create Or Replace Procedure altaCita (
  P_medico In Cita.medico%Type, 
  P_enfermero In Cita.enfermero%Type, 
  P_paciente In Cita.paciente%Type,   
  P_medio In Cita.medio%Type, 
  P_patologia In Cita.patologia%Type, 
  P_fechaHora In Cita.fechaHora%Type,
  P_asistencia In Cita.asistencia%Type,
  P_urgencia In Cita.urgencia%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_paciente VARCHAR(10);

Begin
SELECT DNI INTO DNI_paciente FROM Paciente WHERE id = P_paciente;
resultado:='Alta Cita de Paciente con DNI: ' || DNI_paciente || ' correcta.';
Insert Into Cita(medico,enfermero,paciente,medio,patologia,fechaHora,asistencia,urgencia) Values (P_medico,P_enfermero,P_paciente,P_medio,P_patologia,P_fechaHora,P_asistencia,P_urgencia);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: Ya existe una Cita con el mismo cdigo para este Paciente.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica una Cita de un Paciente en la BD

Create Or Replace Procedure modCita (
  P_ID In Cita.ID%Type,
  P_medico In Cita.medico%Type, 
  P_enfermero In Cita.enfermero%Type, 
  P_paciente In Cita.paciente%Type,   
  P_medio In Cita.medio%Type, 
  P_patologia In Cita.patologia%Type, 
  P_fechaHora In Cita.fechaHora%Type,
  P_asistencia In Cita.asistencia%Type,
  P_urgencia In Cita.urgencia%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT DNI INTO DNI_paciente FROM paciente WHERE id = P_paciente;
resultado:='Modificacin de Cita de Paciente con DNI: ' || DNI_paciente || ' correcta.';
Update Cita Set medico= P_medico, enfermero= P_enfermero, paciente= P_paciente, medio= P_medio, patologia= P_patologia, fechaHora= P_fechaHora, asistencia= P_asistencia, urgencia= P_urgencia Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe ese cdigo de Cita para este Paciente.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina una Cita de un Paciente de la BD.

Create Or Replace Procedure bajaCita (
  P_ID In Cita.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT paciente INTO P_paciente FROM Cita WHERE ID = P_ID;
SELECT DNI INTO DNI_paciente FROM paciente WHERE ID = P_paciente;
resultado:='Eliminacin de Cita con ID: ' || P_ID || ' del Paciente con DNI: ' || DNI_paciente || ' correcta.';
Delete From Cita Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe ese cdigo de Cita para este Paciente.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/


-- Creacin del Procedimiento que aade un Tratamiento a la Patologa de un Paciente en la BD.

create or replace 
Procedure altaTratamiento (
  P_patologia In Tratamiento.patologia%Type, 
  P_farmaco In Tratamiento.farmaco%Type,
  P_medico In Tratamiento.medico%Type,
  P_fechaInicio In Tratamiento.fechaInicio%Type,
  P_fechaFin In Tratamiento.fechaInicio%Type,
  P_periodicidad In Tratamiento.periodicidad%Type,
  P_cantidad In Tratamiento.cantidad%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
OK_alergia Integer;
P_error_alergia exception;
cursor C_alergias(ID_Paciente Integer) is
  SELECT * FROM Farmaco WHERE upper(principio) LIKE upper('%' || 
  (SELECT TipoAlergia.denominacion FROM TipoAlergia WHERE TipoAlergia.ID = 
  (SELECT Alergia.tipo FROM Alergia WHERE Alergia.paciente = ID_paciente)) || '%') AND farmaco.id = P_farmaco;
contador number:=0;
P_alergias C_alergias%ROWTYPE;

Begin
SELECT paciente INTO P_paciente FROM Patologia WHERE ID = P_patologia;
SELECT DNI INTO DNI_paciente FROM Paciente WHERE id = P_paciente;
SELECT COUNT(*) INTO OK_alergia FROM Alergia WHERE Alergia.paciente = P_paciente;
resultado:='Alta Tratamiento de Paciente con DNI: ' || DNI_paciente || ' para Patologa con ID: ' || P_patologia || ' correcta.';
OPEN C_alergias(P_paciente);
LOOP FETCH C_alergias INTO P_alergias;
  EXIT WHEN C_alergias%NOTFOUND;
  contador:= contador + 1;
  --dbms_output.put_line ('Contador: ' || contador);
END LOOP;
IF (contador>0) AND (OK_alergia>0) THEN
	Raise P_error_alergia;
END IF;
CLOSE C_alergias;
Insert Into Tratamiento(patologia,farmaco,medico,fechaInicio,fechaFin,periodicidad,cantidad) Values (P_patologia,P_farmaco,P_medico,P_fechaInicio,P_fechaFin,P_periodicidad,P_cantidad);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
  
  When P_error_alergia Then
    P_error := SQLCODE;
    resultado:='Error!: El Paciente es alrgico a ese Frmaco.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: Ya existe una Tratamiento con el mismo cdigo para la Patologa de este Paciente.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
		
   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica un Tratamiento de la Patologa de un Paciente en la BD

Create Or Replace Procedure modTratamiento (
  P_ID In Tratamiento.ID%Type,
  P_patologia In Tratamiento.patologia%Type, 
  P_farmaco In Tratamiento.farmaco%Type,
  P_medico In Tratamiento.medico%Type,  
  P_fechaInicio In Tratamiento.fechaInicio%Type,
  P_fechaFin In Tratamiento.fechaInicio%Type,
  P_periodicidad In Tratamiento.periodicidad%Type,
  P_cantidad In Tratamiento.cantidad%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;
OK_alergia Integer;
P_error_alergia exception;
cursor C_alergias(ID_Paciente Integer) is
  SELECT * FROM Farmaco WHERE upper(principio) LIKE upper('%' || 
  (SELECT TipoAlergia.denominacion FROM TipoAlergia WHERE TipoAlergia.ID = 
  (SELECT Alergia.tipo FROM Alergia WHERE Alergia.paciente = ID_paciente)) || '%') AND farmaco.id = P_farmaco;
contador number:=0;
P_alergias C_alergias%ROWTYPE;

Begin
SELECT paciente INTO P_paciente FROM Patologia WHERE ID = P_patologia;
SELECT DNI INTO DNI_paciente FROM paciente WHERE id = P_paciente;
SELECT COUNT(*) INTO OK_alergia FROM Alergia WHERE Alergia.paciente = P_paciente;
resultado:='Modificacin de Tratamiento de Paciente con DNI: ' || DNI_paciente || ' para Patologa con ID: ' || P_patologia || ' correcta.';
OPEN C_alergias(P_paciente);
LOOP FETCH C_alergias INTO P_alergias;
  EXIT WHEN C_alergias%NOTFOUND;
  contador:= contador + 1;
  --dbms_output.put_line ('Contador: ' || contador);
END LOOP;
IF (contador>0) AND (OK_alergia>0) THEN
	Raise P_error_alergia;
END IF;
CLOSE C_alergias;
Update Tratamiento Set patologia= P_patologia, farmaco= P_farmaco, medico= P_medico, fechaInicio= P_fechaInicio, fechaFin= P_fechaFin, periodicidad= P_periodicidad, cantidad= P_cantidad  Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
	Raise P_error_codigo;
END IF;
Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

   When P_error_alergia Then
    P_error := SQLCODE;
    resultado:='Error!: El Paciente es alrgico a ese Frmaco.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
  
  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe una Tratamiento con el mismo cdigo para la Patologa de este Paciente.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina un Tratamiento de una Patologa de un Paciente de la BD.

Create Or Replace Procedure bajaTratamiento (
  P_ID In Tratamiento.ID%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_patologia Integer;
P_paciente Integer;
DNI_paciente VARCHAR(10);
P_error_codigo exception;

Begin
SELECT patologia INTO P_patologia FROM Tratamiento WHERE ID = P_ID;
SELECT paciente INTO P_paciente FROM Patologia WHERE ID = P_patologia;
SELECT DNI INTO DNI_paciente FROM paciente WHERE ID = P_paciente;
resultado:='Eliminacin de Tratamiento con ID: ' || P_ID || ' de la Patologa con ID: ' || P_patologia || ' y del Paciente con DNI: ' || DNI_paciente || ' correcta.';
Delete From Tratamiento Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe ese cdigo de Tratamiento para la Patologia de ese Paciente.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que aade una Administracion de un Tratamiento en la BD.

Create Or Replace Procedure altaAdministracion (
  P_enfermero In Administracion.enfermero%Type, 
  P_tratamiento In Administracion.tratamiento%Type,   
  P_fecha In Administracion.fecha%Type,
  P_periodo In Administracion.periodo%Type,
  P_cantidad In Administracion.cantidad%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
fecha_Tratamiento date;
periodo_Tratamiento Integer;
cantidad_Tratamiento Integer;
P_error_fecha exception;
P_error_periodo exception;
P_error_cantidad exception;

Begin
SELECT fechaFin INTO fecha_Tratamiento FROM Tratamiento WHERE id = P_tratamiento;
SELECT periodicidad INTO periodo_Tratamiento FROM Tratamiento WHERE id = P_tratamiento;
SELECT cantidad INTO cantidad_Tratamiento FROM Tratamiento WHERE id = P_tratamiento;
resultado:='Alta Administracion de Tratamiento con ID: ' || P_tratamiento || ' correcta.';
IF (P_fecha>fecha_Tratamiento) THEN
	Raise P_error_fecha;
END IF;
IF (P_periodo>periodo_Tratamiento) THEN
	Raise P_error_periodo;
END IF;
IF (P_cantidad!=cantidad_Tratamiento) THEN
	Raise P_error_cantidad;
END IF;
Insert Into Administracion(enfermero,tratamiento,fecha,periodo,cantidad) Values (P_enfermero,P_tratamiento,P_fecha,P_periodo,P_cantidad);
P_error:= SQLCODE;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When P_error_fecha Then
    P_error := SQLCODE;
    resultado:='Error!: El Tratamiento ya ha finalizado: ' || fecha_Tratamiento;
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_periodo Then
    P_error := SQLCODE;
    resultado:='Error!: Periodo de administracin de tratamiento incorrecto, el mximo de periodos es: ' || periodo_Tratamiento;
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_cantidad Then
    P_error := SQLCODE;
    resultado:='Error!: La cantidad prescrita en el Tratamiento es de: ' || cantidad_Tratamiento;
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit; 	
	
  When NO_DATA_FOUND Then
    P_error:= SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When DUP_VAL_ON_INDEX Then
    P_error:= SQLCODE;
    resultado:='Error!: Ya existe una Administracion con el mismo cdigo para este Tratamiento.';
    Dbms_output.Put_line(resultado);
    rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que modifica una Administracion de un Tratamiento en la BD

Create Or Replace Procedure modAdministracion (
  P_ID In Administracion.ID%Type,
  P_enfermero In Administracion.enfermero%Type, 
  P_tratamiento In Administracion.tratamiento%Type,   
  P_fecha In Administracion.fecha%Type,
  P_periodo In Administracion.periodo%Type,
  P_cantidad In Administracion.cantidad%Type,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
fecha_Tratamiento date;
periodo_Tratamiento Integer;
cantidad_Tratamiento Integer;
P_error_fecha exception;
P_error_periodo exception;
P_error_cantidad exception;
P_error_codigo exception;

Begin
SELECT fechaFin INTO fecha_Tratamiento FROM Tratamiento WHERE id = P_tratamiento;
SELECT periodicidad INTO periodo_Tratamiento FROM Tratamiento WHERE id = P_tratamiento;
SELECT cantidad INTO cantidad_Tratamiento FROM Tratamiento WHERE id = P_tratamiento;
resultado:='Modificacin de Administracion de Tratamiento con ID: ' || P_tratamiento || ' correcta.';
IF (P_fecha>fecha_Tratamiento) THEN
	Raise P_error_fecha;
END IF;
IF (P_periodo>periodo_Tratamiento) THEN
	Raise P_error_periodo;
END IF;
IF (P_cantidad!=cantidad_Tratamiento) THEN
	Raise P_error_cantidad;
END IF;
Update Administracion Set enfermero= P_enfermero, tratamiento= P_tratamiento, fecha= P_fecha, periodo= P_periodo, cantidad= P_cantidad Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
   Raise P_error_codigo;
END IF;

Commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception

  When P_error_fecha Then
    P_error := SQLCODE;
    resultado:='Error!: El Tratamiento ya ha finalizado: ' || fecha_Tratamiento;
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_periodo Then
    P_error := SQLCODE;
    resultado:='Error!: Periodo de administracin de tratamiento incorrecto, el mximo de periodos es: ' || periodo_Tratamiento;
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_cantidad Then
    P_error := SQLCODE;
    resultado:='Error!: La cantidad prescrita en el Tratamiento es de: ' || cantidad_Tratamiento;
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit; 	
	 	
  When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe ese cdigo de Administracion para este tratamiento.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/

-- Creacin del Procedimiento que elimina una Administracion de un Tratamiento de la BD.

Create Or Replace Procedure bajaAdministracion (
  P_ID In Administracion.ID%Type,
  Resultado Out Nocopy Varchar)
Is
P_error Integer;
P_error_codigo exception;
P_tratamiento Integer;

Begin
SELECT tratamiento INTO P_tratamiento FROM Administracion WHERE ID = P_ID;
resultado:='Eliminacin de la Administracion con ID: ' || P_ID || ' del Tratamiento con ID : ' || P_tratamiento || ' correcta.';
Delete From Administracion Where ID= P_ID;
P_error := SQLCODE;
IF (SQL%ROWCOUNT=0) Then
    Raise P_error_codigo;
END IF;
commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Dbms_output.Put_line(resultado);

Exception
 
 When NO_DATA_FOUND Then
    P_error := SQLCODE;
    resultado:='Error!: El cdigo no existe.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;

  When P_error_codigo Then
    P_error := SQLCODE;
    resultado:='Error!: No existe ese cdigo de Administracion para este Tratamiento.';
    Dbms_output.Put_line(resultado);
    Rollback;
    Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
    commit;
   
  When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
End;
/