/* 
Script de creacin del esquema del DW (tablas y secuencias).
PFC - Base de dades
Autor: Alfredo del Valle Lpez
*/
-- Eliminar tablas a crear si existen previamente

Drop Table DWPaciente Cascade Constraints;
Drop Table DWPeriodo Cascade Constraints;
Drop Table DWCentro Cascade Constraints;
Drop Table DWMedico Cascade Constraint;
Drop Table DWCita Cascade Constraints;
Drop Table DWProvincia Cascade Constraints;
Drop Table DWPrueba Cascade Constraints;
Drop Table DWEnfermero Cascade Constraints;
Drop Table DWPatologia Cascade Constraints;
Drop Table DWProveedor Cascade Constraints;
Drop Table DWFarmaceutico Cascade Constraints;
Drop Table DWFarmaco Cascade Constraints;
Drop Table DWPedido Cascade Constraints;
Drop Table DWAdministracion Cascade Constraints;
Drop Table DWTratamiento Cascade Constraints;

-- Eliminar secuencias a crear si existen previamente

Drop Sequence Seq_DWPeriodo;

--Crear tablas

Create Table DWPaciente(
    ID Integer Constraint PK_DWPACIENTE Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_DWPACIENTE UNIQUE,
    nombre Varchar(50 Char) Not Null,
    apellidos Varchar(100 Char) Not Null
);

Create Table DWPeriodo(
    ID Integer Constraint PK_DWPERIODO Primary key,
    mes Integer Not Null,
    trimestre Integer Not Null,
    anyo Integer Not Null
);

Create Table DWCentro (
    ID Integer Constraint PK_DWCENTRO Primary Key,
	descripcion Varchar(100 Char) Not Null
);

Create Table DWMedico(
    ID Integer Constraint PK_DWMEDICO Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_DWMEDICO UNIQUE,
    nombre Varchar(50 Char) Not Null,
    apellidos Varchar(100 Char) Not Null
);

Create Table DWPatologia(
    ID Integer Constraint PK_DWPATOLOGIA Primary Key,
    diagnostico Varchar(100 Char) Not Null,
	fechaInicio Date Not Null,
    fechaFin Date,
    baja number(1) check(baja in (0, 1)) Not Null
);

Create Table DWProvincia (
    ID Integer Constraint PK_DWPROVINCIA Primary Key,
	descripcion Varchar(50 Char) Not Null   
);

Create Table DWPrueba(
    ID Integer Constraint PK_DWPRUEBA Primary Key,
    descripcion Varchar(70 Char) Not Null,
	fechaHora Date Not Null
);

Create Table DWEnfermero(
    ID Integer Constraint PK_DWENFERMERO Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_DWENFERMERO UNIQUE,
    nombre Varchar(50 Char) Not Null,
    apellidos Varchar(100 Char) Not Null
);

Create Table DWCita(
	provincia Integer Constraint FK1_DWCITA References DWProvincia(ID) Not Null,
	enfermero Integer Constraint FK2_DWCITA References DWEnfermero(ID) Not Null,    
	patologia Integer Constraint FK3_DWCITA References DWPatologia(ID) Not Null,		
	periodo Integer Constraint FK4_DWCITA References DWPeriodo(ID) Not Null,
    centro Integer Constraint FK5_DWCITA References DWCentro(ID) Not Null,
	medico Integer Constraint FK6_DWCITA References DWMedico(ID) Not Null,
    paciente Integer Constraint FK7_DWCITA References DWPaciente(ID) Not Null,		
	medio Varchar(50 Char) Not Null,	
    fechaHora Date Not Null,
    asistencia number(1) check(asistencia in (0, 1)) Not Null,
    urgencia number(1) check(urgencia in (0, 1)) Not Null, 
	numCitas Integer Default 0 Not Null,
	CONSTRAINT PK_DWCITA PRIMARY KEY (provincia, enfermero, patologia, periodo, centro, medico, paciente)   
);

Create Table DWProveedor (
    ID Integer Constraint PK_DWPROVEEDOR Primary Key,
    CIF Varchar(10 Char) Not null CONSTRAINT AK_DWPROVEEDOR UNIQUE, 
    descripcion Varchar(50 Char) Not Null
);

Create Table DWFarmaceutico(
    ID Integer Constraint PK_DWFARMACEUTICO Primary Key,
    DNI Varchar(10 Char) Not null CONSTRAINT AK_DWFARMACEUTICO UNIQUE,
    nombre Varchar(50 Char) Not Null,
    apellidos Varchar(100 Char) Not Null
);

Create Table DWFarmaco (
    ID Integer Constraint PK_DWFARMACO Primary Key,
    descripcion Varchar(100 Char) Not Null,
    principio Varchar(75 Char) Not Null
);

Create Table DWPedido(
    periodo Integer Constraint FK1_DWPEDIDO References DWPeriodo(ID) Not Null,
    farmaco Integer Constraint FK2_DWPEDIDO References DWFarmaco(ID) Not Null,
	farmaceutico Integer Constraint FK3_DWPEDIDO References DWFarmaceutico(ID) Not Null,
    centro Integer Constraint FK4_DWPEDIDO References DWCentro(ID) Not Null,
	proveedor Integer Constraint FK5_DWPEDIDO References DWProveedor(ID) Not Null,
    cantidad Integer Not Null,
	fechaHora Date Not Null,
	numPedidos Integer Default 0 Not Null, 
	CONSTRAINT PK_DWPEDIDO PRIMARY KEY (periodo, farmaco, farmaceutico, centro, proveedor, fechaHora)  
);

Create Table DWTratamiento(
    ID Integer Constraint PK_DWTRATAMIENTO Primary Key,
	fechaInicio Date Not Null,    
	fechaFin Date Not Null,
	periodicidad Integer Not Null,
	cantidad Integer Not Null
);

Create Table DWAdministracion(
    enfermero Integer Constraint FK1_DWADMINISTRACION References DWEnfermero(ID) Not Null,
	periodo Integer Constraint FK2_DWADMINISTRACION References DWPeriodo(ID) Not Null,	
	patologia Integer Constraint FK3_DWADMINISTRACION References DWPatologia(ID) Not Null,	
	tratamiento Integer Constraint FK5_DWADMINISTRACION References DWTratamiento(ID) Not Null,	
    farmaco Integer Constraint FK6_DWADMINISTRACION References DWFarmaco(ID) Not Null,
    centro Integer Constraint FK7_DWADMINISTRACION References DWCentro(ID) Not Null,		
	fecha Date Not Null,    
	turno Integer Not Null,
	cantidad Integer Not Null,	
	numAdministraciones Integer Default 0 Not Null,
	CONSTRAINT PK_DWADMINISTRACION PRIMARY KEY (enfermero, periodo, patologia, tratamiento, farmaco, centro, fecha, turno)  
);


--Crear secuencias
        
Create Sequence Seq_DWPeriodo Increment By 1 Start With 1;


--Iniciar secuencias

Create Or Replace Trigger Insert_ID_DWPeriodo
    Before Insert On DWPeriodo
    For Each Row
Begin
   Select Seq_DWPeriodo.Nextval Into :New.ID
   From Dual;
End Insert_ID_DWPeriodo;
/