/* 
Script de creacin del Procedimiento ETL del DW.
PFC - Base de dades
Autor: Alfredo del Valle Lpez
*/

-- Creacin del Procedimiento que traspasa los datos de la BD al DW.

Create Or Replace Procedure ETLDW (
  P_mes In Integer,
  P_anyo In Integer,
  resultado Out Nocopy Varchar)
Is
P_error Integer;
P_trimestre Integer;
P_contador Integer;
P_IDPeriodo DWPeriodo.ID%Type;
P_Cmes Integer;
P_Canyo Integer;
P_Cprovincia DWProvincia.ID%Type; 
P_Cenfermero DWEnfermero.ID%Type; 
P_Cpatologia DWPatologia.ID%Type; 
P_Ccentro DWCentro.ID%Type; 
P_Cmedico DWMedico.ID%Type; 
P_Cpaciente DWPaciente.ID%Type; 
P_Cmedio DWCita.medio%Type; 
P_CfechaHora DWCita.fechaHora%Type; 
P_Casistencia DWCita.asistencia%Type; 
P_Curgencia DWCita.urgencia%Type;
P_contCita Integer;
P_Ctratamiento DWTratamiento.ID%Type; 
P_Cfarmaco DWFarmaco.ID%Type; 
P_Cfecha DWAdministracion.fecha%Type; 
P_Cturno DWAdministracion.turno%Type; 
P_Ccantidad DWAdministracion.cantidad%Type;
P_contAdministracion Integer;
P_Cfarmaceutico DWFarmaceutico.ID%Type;
P_Cproveedor DWProveedor.ID%Type;
P_contPedido Integer;

-- Cursor que almacena las citas de los pacientes por mes.
CURSOR CCitas(C_mes Integer, C_anyo Integer) IS
 Select Extract (Month from CI.fechaHora) AS mes, Extract (Year from CI.fechaHora) AS anyo,
        PA.provincia, CI.enfermero, CI.patologia, E.centro, CI.medico, CI.paciente, CI.medio, CI.fechaHora, CI.asistencia, CI.urgencia,
        Count(*) as contCita
    From provincia PO, enfermero E, patologia P, servicioCentro C, medico M, paciente PA, medioCita MC, cita CI  Where
    PA.provincia= PO.ID And
    CI.enfermero= E.ID And
    CI.patologia= P.ID And
    E.centro= C.ID And
    CI.medico= M.ID And
    CI.paciente= PA.ID And
    CI.medio = MC.ID And
    Extract (Month from CI.fechaHora) = P_mes And
    Extract (Year from CI.fechaHora) = P_anyo
    Group By Extract (Month from CI.fechaHora), Extract (Year from CI.fechaHora), PA.provincia, CI.enfermero, CI.patologia, E.centro, CI.medico, CI.paciente, CI.medio, CI.fechaHora, CI.asistencia, CI.urgencia
    Order By mes,anyo,CI.paciente,E.centro;

-- Cursor que almacena las administraciones de los medicamentos por mes.
CURSOR CAdministraciones(C_mes Integer, C_anyo Integer) IS
 Select Extract (Month from A.fecha) AS mes, Extract (Year from A.fecha) AS anyo,
        A.enfermero, T.patologia, A.tratamiento, T.farmaco, E.centro, A.fecha, A.periodo, A.cantidad,
        Count(*) as contAdministracion
    From patologia P, tratamiento T, enfermero E, administracion A, farmaco F, servicioCentro C  Where
    A.enfermero= E.ID And
    T.patologia= P.ID And
	A.tratamiento= T.ID And
    T.farmaco= F.ID And
    E.centro= C.ID And
    Extract (Month from A.fecha) = P_mes And
    Extract (Year from A.fecha) = P_anyo
    Group By Extract (Month from A.fecha), Extract (Year from A.fecha), A.enfermero, T.patologia, A.tratamiento, T.farmaco, E.centro, A.fecha, A.periodo, A.cantidad
    Order By mes,anyo,A.enfermero,E.centro;
	
-- Cursor que almacena los pedidos de los medicamentos por mes.
CURSOR CPedidos(C_mes Integer, C_anyo Integer) IS
 Select Extract (Month from P.fechaHora) AS mes, Extract (Year from P.fechaHora) AS anyo,
        P.farmaco, P.farmaceutico, FA.centro, P.proveedor, P.cantidad, P.fechaHora,
        Count(*) as contPedido
    From pedido P, farmaceutico FA, farmaco F, servicioCentro C, proveedor PO  Where
    P.farmaco= F.ID And
    P.farmaceutico= FA.ID And
	FA.centro= C.ID And
    P.proveedor= PO.ID And
    Extract (Month from P.fechaHora) = P_mes And
    Extract (Year from P.fechaHora) = P_anyo
    Group By Extract (Month from P.fechaHora), Extract (Year from P.fechaHora), P.farmaco, P.farmaceutico, FA.centro, P.proveedor, P.cantidad, P.fechaHora
    Order By mes,anyo,P.farmaco,FA.centro;
	
Begin
P_trimestre := Trunc((P_mes+2)/3);
resultado:= 'Traspaso Mes '|| P_mes ||', Trimestre '|| P_trimestre ||', Ao '|| P_anyo ||' en curso...';
Dbms_output.Put_line(resultado);
resultado:= 'Procedimiento ETL del Mes '|| P_mes ||', Trimestre '|| P_trimestre ||', Ao '|| P_anyo ||' realizado correctamente.';

-- Obtener el identificador del periodo correspondiente al mes y ao especificado. Hay que verificar si el periodo ya existe en el DW, sino crearlo.
Select count(ID) into P_contador From DWPeriodo Where mes=P_mes And anyo=P_anyo;
IF (P_contador=0) THEN
	Insert Into DWPeriodo(mes,trimestre,anyo) Values (P_mes,P_trimestre,P_anyo) RETURNING ID Into P_IDPeriodo;
ELSE
	Select ID into P_IDPeriodo From DWPeriodo Where mes=P_mes And anyo=P_anyo; 
END IF;

-- Resetear los datos de las tablas de hecho por si ya han sido traspasados anteriormente
DELETE FROM DWCita WHERE periodo=P_IDPeriodo;
DELETE FROM DWAdministracion WHERE periodo=P_IDPeriodo;
DELETE FROM DWPedido WHERE periodo=P_IDPeriodo;

-- Insertar nuevos datos en la tabla de dimensin de DWPaciente
Insert Into DWPaciente(ID,DNI,nombre,apellidos) 
Select paciente.ID, paciente.DNI, paciente.nombre, paciente.apellido1 || ' ' || paciente.apellido2  
from paciente Left JOIN DWPaciente ON paciente.ID=DWPaciente.ID Where DWPaciente.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWCentro
Insert Into DWCentro(ID,descripcion) 
Select serviciocentro.ID, (Select centro.denominacion from centro Where centro.ID = serviciocentro.centro) 
from serviciocentro Left JOIN DWCentro ON serviciocentro.ID=DWCentro.ID Where DWCentro.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWMedico
Insert Into DWmedico(ID,DNI,nombre,apellidos) 
Select medico.ID, medico.DNI, medico.nombre, medico.apellido1 || ' ' || medico.apellido2  
from medico Left JOIN DWMedico ON medico.ID=DWMedico.ID Where DWMedico.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWPatologia
Insert Into DWPatologia(ID,diagnostico,fechaInicio, fechaFin,baja) 
Select Patologia.ID, (SELECT denominacion from diagnostico WHERE patologia.diagnostico=diagnostico.ID), patologia.fechaInicio, patologia.fechaFin, patologia.baja  
from patologia Left JOIN DWPatologia ON patologia.ID=DWPatologia.ID Where DWPatologia.ID IS NULL;
	
-- Insertar nuevos datos en la tabla de dimensin de DWProvincia
Insert Into DWProvincia(ID,descripcion) 
Select provincia.ID, provincia.denominacion 
from provincia Left JOIN DWProvincia ON provincia.ID=DWProvincia.ID Where DWProvincia.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWPrueba
Insert Into DWPrueba(ID,descripcion,fechaHora) 
Select prueba.ID, (SELECT denominacion from tipoPrueba WHERE prueba.tipo=tipoPrueba.ID), prueba.fechaHora  
from prueba Left JOIN DWPrueba ON prueba.ID=DWPrueba.ID Where DWPrueba.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWEnfermero
Insert Into DWEnfermero(ID,DNI,nombre,apellidos) 
Select enfermero.ID, enfermero.DNI, enfermero.nombre, enfermero.apellido1 || ' ' || enfermero.apellido2  
from enfermero Left JOIN DWEnfermero ON enfermero.ID=DWEnfermero.ID Where DWEnfermero.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWFarmaco
Insert Into DWFarmaco(ID,descripcion,principio) 
Select farmaco.ID, farmaco.denominacion, farmaco.principio  
from farmaco Left JOIN DWFarmaco ON farmaco.ID=DWFarmaco.ID Where DWFarmaco.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWTratamiento
Insert Into DWTratamiento(ID,fechaInicio,fechaFin,periodicidad,cantidad) 
Select tratamiento.ID, tratamiento.fechaInicio, tratamiento.fechaFin, tratamiento.periodicidad, tratamiento.cantidad  
from tratamiento Left JOIN DWTratamiento ON tratamiento.ID=DWTratamiento.ID Where DWTratamiento.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWFarmaceutico
Insert Into DWFarmaceutico(ID,DNI,nombre,apellidos) 
Select farmaceutico.ID, farmaceutico.DNI, farmaceutico.nombre, farmaceutico.apellido1 || ' ' || farmaceutico.apellido2  
from farmaceutico Left JOIN DWFarmaceutico ON farmaceutico.ID=DWFarmaceutico.ID Where DWFarmaceutico.ID IS NULL;

-- Insertar nuevos datos en la tabla de dimensin de DWProveedor
Insert Into DWProveedor(ID,CIF,descripcion) 
Select proveedor.ID, proveedor.CIF, proveedor.denominacion  
from proveedor Left JOIN DWProveedor ON proveedor.ID=DWProveedor.ID Where DWProveedor.ID IS NULL;


-- Insertar datos del periodo establecido en la tabla de hechos de DWCita
OPEN CCitas(P_mes,P_anyo);
    LOOP
        FETCH Ccitas INTO P_Cmes, P_Canyo, P_Cprovincia, P_Cenfermero, P_Cpatologia, P_Ccentro, P_Cmedico, P_Cpaciente, P_Cmedio, P_CfechaHora, P_Casistencia, P_Curgencia, P_contCita;
        EXIT WHEN Ccitas%NOTFOUND;      
		UPDATE DWCita SET numCitas=numCitas + P_contCita
			Where periodo=P_IDPeriodo AND provincia=P_Cprovincia AND enfermero=P_Cenfermero AND patologia=P_Cpatologia AND centro=P_Ccentro AND medico=P_Cmedico 
			  AND paciente=P_Cpaciente AND medio=P_Cmedio AND fechaHora=P_CfechaHora AND asistencia=P_Casistencia AND urgencia=P_Curgencia;
        IF (SQL%ROWCOUNT=0) Then
			INSERT INTO DWCita (periodo, provincia, enfermero, patologia, centro, medico, paciente, medio, fechaHora, asistencia, urgencia, numCitas)
				VALUES (P_IDPeriodo, P_Cprovincia, P_Cenfermero, P_Cpatologia, P_Ccentro, P_Cmedico, P_Cpaciente, P_Cmedio, P_CfechaHora, P_Casistencia, P_Curgencia, P_contCita);
        END IF; 
    END LOOP;
CLOSE CCitas;

-- Insertar datos del periodo establecido en la tabla de hechos de DWAdministracion
OPEN CAdministraciones(P_mes,P_anyo);
    LOOP
        FETCH CAdministraciones INTO P_Cmes, P_Canyo, P_Cenfermero, P_Cpatologia, P_Ctratamiento, P_Cfarmaco, P_Ccentro, P_Cfecha, P_Cturno, P_Ccantidad, P_contAdministracion;
        EXIT WHEN CAdministraciones%NOTFOUND;      
		UPDATE DWAdministracion SET numAdministraciones=numAdministraciones + P_contAdministracion
			Where periodo=P_IDPeriodo AND enfermero=P_Cenfermero AND patologia=P_Cpatologia AND tratamiento=P_Ctratamiento AND farmaco=P_Cfarmaco AND centro=P_Ccentro  
			  AND fecha=P_Cfecha AND turno=P_Cturno AND cantidad=P_Ccantidad;
       IF (SQL%ROWCOUNT=0) Then
		INSERT INTO DWAdministracion (enfermero, periodo, patologia, tratamiento, farmaco, centro, fecha, turno, cantidad, numAdministraciones)
				VALUES (P_Cenfermero, P_IDPeriodo, P_Cpatologia, P_Ctratamiento, P_Cfarmaco, P_Ccentro, P_Cfecha, P_Cturno, P_Ccantidad, P_contAdministracion);
       END IF; 
    END LOOP;
CLOSE CAdministraciones;

-- Insertar datos del periodo establecido en la tabla de hechos de DWPedido
OPEN CPedidos(P_mes,P_anyo);
    LOOP
        FETCH CPedidos INTO P_Cmes, P_Canyo, P_Cfarmaco, P_Cfarmaceutico, P_Ccentro, P_Cproveedor, P_Ccantidad, P_CfechaHora, P_contPedido;
        EXIT WHEN CPedidos%NOTFOUND;      
		UPDATE DWPedido SET numPedidos=numPedidos + P_contPedido
			Where periodo=P_IDPeriodo AND farmaco=P_Cfarmaco AND farmaceutico=P_Cfarmaceutico AND centro=P_Ccentro AND proveedor=P_Cproveedor 
			  AND cantidad=P_Ccantidad AND fechaHora=P_CfechaHora;
       IF (SQL%ROWCOUNT=0) Then
		INSERT INTO DWPedido (periodo, farmaco, farmaceutico, centro, proveedor, cantidad, fechaHora, numPedidos)
				VALUES (P_IDPeriodo, P_Cfarmaco, P_Cfarmaceutico, P_Ccentro, P_Cproveedor, P_Ccantidad, P_CfechaHora, P_contPedido);
       END IF; 
    END LOOP;
CLOSE CPedidos;

commit;
Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, 0, resultado, SYSTIMESTAMP);
commit;
Exception
   When Others Then
     P_error:= SQLCODE;
     resultado:= 'Error!: ' || P_error ||' ' || SQLERRM;
     Dbms_output.Put_line(resultado);
     rollback;
     Insert into logs(procedimiento,error,descripcion,fecha) Values ($$PLSQL_UNIT, P_error, resultado, SYSTIMESTAMP);
     commit;
	 
End;
/

